### prepare the root by creating basic devices and directories
prep: check
	@if test ! -f $(BITCHESROOT)/state/prep; then \
          echo "Preparing root ..."; \
          make lock; \
          mkdir -p $(ROOT)/var/lib/rpm; \
          chmod g+s $(ROOT)/var/lib/rpm; \
          mkdir -p $(ROOT)/dev; \
          $(MKNOD) $(ROOT)/dev/null c 1 3; \
          mkdir -p $(ROOT)/etc; \
	  cp $(BITCHESROOT)/etc/resolv.conf $(ROOT)/etc; \
	  cp $(BITCHESROOT)/etc/hosts $(ROOT)/etc; \
	  touch $(ROOT)/etc/mtab; \
	  touch $(ROOT)/etc/fstab; \
          mkdir -p $(ROOT)/tmp $(ROOT)/var/tmp; \
          rpm --root $(ROOT) --initdb; \
          touch $(BITCHESROOT)/state/prep; \
          make unlock; \
	else \
	  echo "prep already made."; \
	fi

#for pkg in `cat manifest/base | perl -pe 's@^@../../packages/@' | perl -pe 's/\n/ /g'`; do \
          echo "installing $$pkg"; \
	  $(RPM) --root $(ROOT) -Uhv $$pkg --nodeps || exit 1; \
          done; \### install all packages from the dist's manifest/base dir if not done yet

base: prep
	@if test ! -f $(BITCHESROOT)/state/base; then \
	  echo "Installing base ..."; \
	  make lock; \
	  $(RPM) --root $(ROOT) -Uhv `cat manifest/base | perl -pe 's@^@../../packages/@' | perl -pe 's/\n/ /g'`  --nodeps --noorder || exit 1; \
	  touch $(BITCHESROOT)/state/base; \
	  echo "DISTRO=$(DIST)" > $(BITCHESROOT)/state/dist.mk; \
	  make unlock; \
	else \
	  echo "base already made."; \
	fi

### install all packages from apt if apt has not been made yet
apt: base
	@if test ! -f $(BITCHESROOT)/state/apt; then \
	  echo "Installing apt ..."; \
	  make lock; \
	  $(RPM) --root $(ROOT) -Uhv `cat manifest/apt | perl -pe 's@^@../../packages/@' | perl -pe 's/\n/ /g'`  --nodeps; \
	  touch $(BITCHESROOT)/state/apt; \
	  make unlock; \
	else \
	  echo "apt already made."; \
	fi

### install all stuff necessary for rpm -tb to work
# FIXME: the ugly workaround to create rpmmacros is because I can't figure
# out a way to make a redirect happen in the subshell, inside the chroot
# ie $(CHROOT) $(ROOT) echo "%_topdir /usr/src/rpm" > $(ROOT)/root/.rpmmacros
rpm-build: root
	@if test ! -f $(BITCHESROOT)/state/rpm-build; then \
	  echo "Installing rpm-build ..."; \
	  make lock; \
	  $(CHROOT) $(ROOT) apt-get install -y rpm-devel rpm-build make gcc tar gzip patch $(RPM_BUILD); \
	  echo "%_topdir /usr/src/rpm" > $(ROOT)/tmp/macros; \
	  $(CHROOT) $(ROOT) mv /tmp/macros /etc/rpm; \
          for a in RPMS SRPMS SOURCES SPECS BUILD; do \
	    $(CHROOT) $(ROOT) mkdir -p /usr/src/rpm/$$a; \
          done; \
	  touch $(BITCHESROOT)/state/rpm-build; \
	  make unlock; \
	else \
	  echo "rpm-build already made."; \
	fi

### do the final stuff to make the root ready
### mount the apt-archives and proc binded into the root
root: apt sources.list
	@if test ! -f $(BITCHESROOT)/state/ready; then \
          echo "Readying root ..."; \
          make lock; \
	  make umount; \
	  $(CHROOT) $(ROOT) rm -rf /var/cache/apt/archives/*; \
	  make mount-apt-archives; \
	  make mount-proc; \
	  $(CHROOT) $(ROOT) rm -rf /var/lib/rpm/__*; \
	  $(CHROOT) $(ROOT) apt-get update && \
	  $(CHROOT) $(ROOT) apt-get install -f -y && \
          touch $(BITCHESROOT)/state/ready && \
          make unlock; \
        else \
          echo "Root ready already."; \
        fi

mount-proc:
	  @$(MOUNT) -t proc proc $(ROOT)/proc
	  @echo "$(ROOT)/proc" >> $(BITCHESROOT)/state/mount

mount-apt-archives:
	  @$(MOUNT) -o bind $(BITCHESROOT)/apt-archives $(ROOT)/var/cache/apt/archives
	  @echo "$(ROOT)/var/cache/apt/archives" >> $(BITCHESROOT)/state/mount

### perform various checks on our sanity
check: check-manifest check-dirs check-etc
	@echo "Pre-flight check ok."

### try to make each file mentioned in the manifests into the packages dir
make-manifest:
	@for a in manifest/*; do \
	  if test -f $$a; then \
	    for b in `cat $$a | perl -pe 's/\n/ /g'`; do \
              if test ! -f $(BITCHESROOT)/packages/$$b; then \
		if test ! -z $(CDROM); then \
		  cp $(CDROM)/`basename $$b` $(BITCHESROOT)/packages/$$b; \
		else \
		  cd $(BITCHESROOT)/packages && make $$b; \
		fi; \
              fi; \
            done; \
	  fi; \
	done

### check if the packages listed in the manifest files are available
check-manifest:
	@for a in manifest/*; do \
	  if test -f $$a; then \
	    for b in `cat $$a | perl -pe 's/\n/ /g'`; do \
	      if test ! -f $(BITCHESROOT)/packages/$$b; then \
	        echo "$$b not found in"; \
	        echo "    $(BITCHESROOT)/packages (from $$a)"; \
		FAILED=true; \
	      fi; \
	    done; \
	    if test "x$$FAILED" = "xtrue"; then \
	      echo "Packages not found.  Try running \"make make-manifest\""; \
              exit 1; \
            fi; \
	  fi; \
	done

check-dirs:
	@mkdir -p $(BITCHESROOT)/apt-archives/partial
	@mkdir -p $(ROOT)

check-etc:
	@if test ! -f $(BITCHESROOT)/etc/resolv.conf; then \
	  echo "Please create $(BITCHESROOT)/etc/resolv.conf !"; \
	  exit 1; \
	fi
	@if test ! -f $(BITCHESROOT)/etc/hosts; then \
	  echo "Please create $(BITCHESROOT)/etc/hosts !"; \
	  exit 1; \
	fi
